#//-----------------------------------------------
#//TUSB-K02ADVZ用Pythonサンプルプログラム
#// 処理関数等
#//
#//2024_04_23　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import tkinter as tk
import tkinter.ttk as ttk
import threading
from tkinter import messagebox
import time

def ShowInfo(str):
    """文字列を表示する"""
    messagebox.showinfo("",str)

def ForceToInt(nstr,err=-1):
    """文字列をintで返す。他の型でも可能な限りintに変換して返す。変換できなくても何か返す。
    
    Arguments:
    str         ----    変換したい文字列
    err         ----    変換できない時に返す値(デフォルト-1)
    
    Return:
    int         ----    変換した値またはエラー指定値
    """
    if type(nstr) is int: return nstr
    if type(nstr) is float:
        try:
            return int(nstr)
        except Exception:
            return err
    if (type(nstr) is str):
        try:
            if nstr.isdecimal() : return int(nstr)
            return int(float(nstr))
        except Exception:
            return err
    return err

def ForceToFloat(nstr,err=-1.0):
    """文字列をfloatで返す。他の型でも可能な限りfloatに変換して返す。変換できなくても何か返す。
    
    Arguments:
    str         ----    変換したい文字列
    err         ----    変換できない時に返す値(デフォルト-1.0)
    
    Return:
    float       ----    変換した値またはエラー指定値
    """
    if type(nstr) is int: return float(nstr)
    if type(nstr) is float: return nstr
    if (type(nstr) is str):
        try:
            return float(nstr)
        except ValueError:
            return err
    return err

def bitChk(dat: int,bit) -> bool:
    """datを2進数としてbit桁目のビットをチェックする

    Reuturns:
    bool            ----    ビットが1ならTrue 0ならFalse
    """
    dat >>= bit
    return (( dat & 1 ) == 1)

def GetLFrame(master,title,relw,relh,relx,rely):
    """ラベル付きフレームを作成する
    
    arguments:
    master      ----    配置ペアレント
    title       ----    フレームのタイトル文字列
    relw        ----    表示幅(相対値)
    relh        ----    表示高(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上端)

    returns:
    LabelFrame  ----    生成されたラベル付きフレームインスタンス
    """
    frame = tk.LabelFrame(master, text=title)
    frame.place(relwidth= relw,relheight=relh,relx=relx,rely=rely,anchor=tk.NW)
    return frame

def GetEntry(master,init,relw,relx,rely,command=None):
    """テキスト入力ボックスを作成する
    
    arguments:
    master      ----    配置ペアレント
    init        ----    初期入力文字列
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    入力文字列が変更された時に呼び出される関数

    returns:
    Entry       ----    生成されたテキストボックスインスタンス
    """
    entry = tk.Entry(master)
    entry.bind("<<TextModified>>",command)
    entry.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    entry.insert(tk.END,init)
    return entry

def GetButton(master,text,command,relw,relx,rely):
    """ボタンを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    ボタンに表示する文字列
    command     ----    ボタンが押された時に呼び出されるハンドラ関数
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左右中央)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Button      ----    生成されたボタンインスタンス
    """
    button = tk.Button(master,text=text,command=command)
    button.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.CENTER)
    return button

def GetComboBox(master,value,relw,relx,rely,command=None):
    """コンボボックスを作成する
    
    arguments:
    master      ----    配置ペアレント
    value       ----    コンボボックスに設定するデータリスト(又はタプル)
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    選択値が変更した時に呼び出される関数(引数としてイベントが渡される)

    returns:
    Combobox    ----    生成されたコンボボックスインスタンス
    """
    combobox = ttk.Combobox(master,value=value,state="readonly")
 #   combobox.bind("<<ComboboxSelected>>",command)
    combobox.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    combobox.current(0)
    return combobox

def GetRadioButtons(master,dic,relx,rely,relxp,relyp,val,command=None):
    """ラジオボタン選択を作成する

    arguments:
    master      ----    配置ペアレント
    dic         ----    ラジオボタンのtextをkey、ラジオボタンのvalueをvalueにセットした辞書型
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    relxp       ----    ラジオボタン間左右ピッチ
    relyp       ----    ラジオボタン間上下ピッチ
    val         ----    variableを対応させるwidget変数(例 IntVar())
    command     ----    ラジオボタン変更時ハンドラ関数

    returns:
    ラジオボタンインスタンスリスト
    """
    ret_rbtn = list()
    cnt = 0
    for key in dic.keys():
        ret_rbtn.append(tk.Radiobutton(master,text=key,value=dic[key],variable=val,command=command))
        ret_rbtn[cnt].place(relx = relx+cnt*relxp,rely = rely+cnt*relyp,anchor=tk.W)
        cnt += 1
    return ret_rbtn

def GetLabel(master,text,relx,rely):
    """ラベルを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    表示する文字列
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成されたラベルインスタンス
    """
    label = tk.Label(master,text=text)
    label.place(relx=relx,rely=rely,anchor=tk.W)
    return label

def GetChkBox(master,text,relx,rely,val):
    """チェックボックスを生成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    表示する文字列
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    val         ----    variableを対応させるwidget変数(例 BooleanVar())

    returns:
    Checkbutton ----    生成されたチェックボックスのインスタンス
    """
    chkbox = tk.Checkbutton(master,text=text,variable=val)
    chkbox.place(relx=relx,rely=rely,anchor=tk.W)
    return chkbox

def GetLabelDisp(master,text,relw,relx,rely):
    """表示用ラベルを生成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    生成時に表示する文字列
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成された表示用ラベルのインスタンス
    """
    label = tk.Label(master,text=text,relief="groove")
    label.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    return label

def GenList(N,initval):
    """list generator with value initialize"""
    ret = list()
    for i in range(N):
        ret.append(initval)
    return ret

def GenTkVarList(N,cls,initval):
    """Tkinter Var list generator with value initialize
    
    arguments:
    N           ----    list length
    cls         ----    tkinter variable class
    initval     ----    initial value
    
    returns:
    list        ----    generated list
    """
    ret = list()
    for i in range(N):
        ret.append(cls(value=initval))
    return ret

# タイマースレッド管理クラス ////////////////////////////////////
#厳密なインターバルではない。関数処理が終わったら指定時間待つだけ。
#from builtins import function
class intTT():
    """タイマースレッド管理クラス
    厳密なインターバル処理ではない。
    関数処理が終わったら指定時間待って再度実行しつづるループ。
    """
    thread :threading.Thread = None # スレッドクラスオブジェクト
    thread_en : bool = None         # スレッド実行イネーブルフラグ
    call_func = None                # ループで実行する関数
    interval = None                 # ループの待ち時間

    def __init__(self,inttime,callf):
        """初期化"""
        self.interval = inttime # インターバルタイム
        self.call_func = callf  # 処理関数登録
    
    def ThreadF(self):
        """スレッド関数(ループ)"""
        while self.thread_en:
            self.call_func()
            time.sleep(self.interval)

    def Start(self):
        """スレッド開始処理"""
        if self.thread == None:
            self.thread_en = True
            self.thread = threading.Thread(target=self.ThreadF)
            self.thread.start()
    
    def Stop(self):
        """スレッド停止処理"""
        if self.thread_en != None:
            self.thread_en = False
            self.thread.join(self.interval+0.1)
            self.thread = None 
